<?php

declare(strict_types=1);

namespace hood\vk\api\methods;

use hood\vk\robots\robot,
    hood\vk\robots\group,
    hood\vk\robots\user;

/**
 * Сообщение
 *
 * @method public static function put(RobotAbstract $from, int $to, string $message, int $mode = 2) Отправить сообщение
 *
 * @see https://vk.com/dev/messages.send
 * @see https://vk.com/dev/messages.getById
 *
 * @package hood\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 *
 * @todo Доработать строгий режим отправки: проверку сообщения в беседе (не имеет ID сообщений)
 */
final class messages extends method
{
    /**
     * @param string $message Сообщение
     */
    protected string $message;

    /**
     * @param int|string|array|null $destination Получатель
     */
    protected $destination;

    /**
     * @param array $attachments Вложения
     */
    protected array $attachments = [];

    /**
     * @param int $mode Режим отправки
     */
    protected int $mode = 1;

    /**
     * Создать сообщение
     *
     * Если переданы все параметры, то сразу отправляет
     *
     * @param robot $robot Робот
     * @param string $message Текст
     * @param int|string|array|null $destination Получатель
     * @param string|null ...$attachments Вложения
     *
     * @return self
     */
    public function __construct(robot $robot, string $message, $destination = null, string ...$attachments)
    {
        // Инициализация параметров

        // Робот
        $this->robot = $robot;

        // Сообщение
        $this->message = $message;

        // Получатель
        if (isset($destination)) {
            $this->destination = $destination;
        } else {
            return;
        }

        // Вложения
        if (isset($attachments)) {
            $this->attachments = $attachments;
        } else {
            return;
        }

        // Отправка, если все параметры инициализированы
        return $this->send($destination);
    }

    /**
     * Отправить сообщение
     *
     * @param int|string|array $destination Получатель
     *
     * @see https://vk.com/dev/messages.send
     *
     * @return array Ответ сервера
     */
    public function send($destination): array
    {
        // Идентификатор сообщения
        $random_id = time();

        if ($this->mode = 1) {
            // Перемножение (по умолчанию)
            $random_id *= rand();
        }

        # Ключ
        if ($this->robot instanceof group) {
            // Робот-группа
            $settings['access_token'] = $this->robot->key;
        } else if ($this->robot instanceof User) {
            // Робот-пользователь
            $settings['access_token'] = $this->robot->key;
        }

        // Версия API
        $settings['v'] = $this->robot->version;

        // Цель отправки
        if (is_int($destination)) {
            // Отправить по идентификатору
            $settings['peer_id'] = $destination;
        } else if (is_array($destination)) {
            // Массовая отправка по идентификаторам
            $settings['user_ids'] = $destination;
        } else {
            // Отправить по домену
            $settings['domain'] = $destination;
        }

        // Сообщение
        $settings['message'] = $this->message;

        // Идентификатор сообщения
        $settings['random_id'] = $random_id;

        // Фильтрация вложений
        $forward_messages = [];
        foreach ($this->attachments as &$attachment) {
            if (iconv_substr($attachment, 0, 7, "UTF-8") === 'message') {
                // Если среди вложений найдено сообщение для пересылки
                $forward_messages[] = $attachment;
                unset($attachment);
            }
        }

        if (!empty($forward_messages)) {
            // Если есть пересылаемые сообщения
            $settings['forward_messages'] = implode(',', $forward_messages);
        }

        if (!empty($attachments)) {
            // Если есть вложения
            $settings['attachment'] = implode(',', $attachments);
        }

        // Запрос
        $request = $this->robot->browser->request('POST', 'messages.send', ['form_params' => $settings]);
        
        // Очистка
        unset($settings);

        if ($this->mode >= 2) {
            // Если установлен режим 2 (усиленная проверка отправленного сообщения)

            if (!empty($request["response"])) {
                // Если пришел ID сообщения

                // Ключ
                if ($this->robot instanceof Group) {
                    // Робот-группа
                    $settings['access_token'] = $this->robot->key;
                } else if ($this->robot instanceof User) {
                    // Робот-пользователь
                    $settings['access_token'] = $this->robot->key;
                }

                // Версия API
                $settings['v'] = $this->robot->version;

                // Запрашиваемые сообщения
                $settings['message_ids'] = $request["response"];

                // Запрос
                if ($this->robot->browser()->post('https://api.vk.com/method/messages.getById', $settings)['response']['count'] === 0) {
                    // Если сообщения не существует, то повторить отправку
                    $this->send($destination);
                }
            } else {
                // Что-то придумать :)
            }
        }

        return (array) $request;
    }

    /**
     * Получить информацию о сообщении
     *
     * @return array Информация о сообщении
     */
    public function info(): array
    {
        # Ключ
        if ($this->robot instanceof group) {
            // Робот-группа
            $settings['access_token'] = $this->robot->key;
        } else if ($this->robot instanceof User) {
            // Робот-пользователь
            $settings['access_token'] = $this->robot->key;
        }

        // Цель отправки
        if (is_int($this->destination)) {
            // Отправить по идентификатору
            $settings['peer_id'] = $this->destination;
        } else if (is_array($this->destination)) {
            // Массовая отправка по идентификаторам
            $settings['user_ids'] = $this->destination;
        } else {
            // Отправить по домену
            $settings['domain'] = $this->destination;
        }

        // Версия API
        $settings['v'] = $this->robot->version;

        // Сообщение
        $settings['message'] = $this->message;

        // Режим отправки
        $settings['mode'] = $this->mode;

        // Фильтрация вложений
        $forward_messages = [];
        foreach ($this->attachments as &$attachment) {
            if (iconv_substr($attachment, 0, 7, "UTF-8") === 'message') {
                // Если среди вложений найдено сообщение для пересылки
                $forward_messages[] = $attachment;
                unset($attachment);
            }
        }

        if (!empty($forward_messages)) {
            // Если есть пересылаемые сообщения
            $settings['forward_messages'] = implode(',', $forward_messages);
        }

        if (!empty($attachments)) {
            // Если есть вложения
            $settings['attachment'] = implode(',', $attachments);
        }

        return $settings;
    }
}
