<?php

declare(strict_types=1);

namespace hood\vk\robots;

use Exception;

use GuzzleHttp\Client as browser;

use hood\vk\core,
    hood\vk\api\methods\method,
    hood\vk\proxies\proxy,
    hood\vk\captcha\captcha;

use hood\accounts\vk as account;

/**
 * Робот
 *
 * @property-read int $id Идентификатор
 * @property-read int $session Сессия
 * @property string $key Ключ
 * @property float $version Версия API
 * @property account $account Аккаунт
 * @property browser $browser Браузер
 * @property proxy $proxy Прокси
 * @property captcha $captcha Обработчик капчи
 *
 * @property int $messages_mode Режим отправки сообщений
 *
 * @method public function __construct(int $id = null, float $version = null) Конструктор
 * @method public function key(string $key) Инициализация ключа
 * @method public function account(account $account) Инициализация аккаунта
 * @method public function __set($name, $value) Запись свойства
 * @method public function __get($name) Чтение свойства
 * @method public function __isset($name) Проверка на инициализированность свойства
 * @method public function __call(string $method, array $params) Вызов метода
 * @method public static function __callStatic(string $method, array $params) Вызов статического метода
 * @method public function __toString() Конвертация в строку
 *
 * @package hood\vk\robots
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 */
abstract class robot
{
    /**
     * @var int Идентификатор
     */
    protected int $id;

    /**
     * @var int Сессия
     */
    protected int $session;

    /**
     * @var string Ключ
     */
    protected string $key;

    /**
     * @var float Версия API
     */
    protected float $version = 5.124;

    /**
     * @var string Аккаунт
     */
    private account $account;

    /**
     * @var proxy Прокси
     */
    protected proxy $proxy;

    /**
     * @var captcha Обработчик капчи
     */
    protected captcha $captcha;

    /**
     * @var int $messages_mode Режим отправки сообщений
     */
    protected int $messages_mode = 1;


    /**
     * Конструктор
     *
     * @param int|null $id Идентификатор
     * @param float|null $version Версия API
     */
    public function __construct(int $id = null, float $version = null)
    {
        // Инициализация ядра
        $core = core::init();

        // Идентификация робота
        $this->id = empty($id) ? $core->robots + 1 : $id;

        // Регистрация робота в ядре
        $core->set($this->id, $this);

        // Идентификация сессии робота
        $this->session = count($core->get($this->id));

        if (isset($version)) {
            // Инициализация версии
            $this->version = $version;
        }
    }

    /**
     * Инициализация ключа
     *
     * @param string $key Ключ
     *
     * @return self
     */
    public function key(string $key): self
    {
        if (!isset($this->key)) {
            $this->key = $key;
        } else {
            throw new Exception('Запрещено перезаписывать ключ');
        }

        return $this;
    }

    /**
     * Инициализация аккаунта
     *
     * @param account $account Аккаунт
     *
     * @return self
     */
    public function account(account $account): self
    {
        if (!isset($this->account)) {
            $this->account = $account;
        } else {
            throw new Exception('Запрещено перезаписывать аккаунт');
        }

        return $this;
    }

    /**
     * Инициализация прокси
     *
     * @param proxy $proxy Прокси
     *
     * @return self
     */
    public function proxy(proxy $proxy): self
    {
        $this->proxy = $proxy;

        return $this;
    }

    /**
     * Инициализация обработчика капчи
     *
     * @param captcha $captcha Обработчик капчи
     *
     * @return self
     */
    public function captcha(captcha $captcha): self
    {
        $this->captcha = $captcha;

        return $this;
    }

    /**
     * Записать свойство
     *
     * @param string $name Название
     * @param mixed $value Значение
     *
     * @return void
     */
    public function __set(string $name, $value): void
    {
        if ($name === 'id') {
            if (!isset($this->id)) {
                $this->id = (int) $value;
                return;
            } else {
                throw new Exception('Запрещено перезаписывать идентификатор');
            }
        } else if ($name === 'session') {
            if (!isset($this->session)) {
                $this->session = (int) $value;
                return;
            } else {
                throw new Exception('Запрещено перезаписывать сессию');
            }
        } else if ($name === 'key') {
            if (!isset($this->key)) {
                $this->key = (string) $value;
                return;
            } else {
                throw new Exception('Запрещено перезаписывать ключ');
            }
        } else if ($name === 'version') {
            if (!isset($this->version)) {
                $this->version = (float) $value;
                return;
            } else {
                throw new Exception('Запрещено перезаписывать версию API');
            }
        } else if ($name === 'account') {
            if (!isset($this->account) && $value instanceof account) {
                $this->account = $value;
                return;
            } else {
                throw new Exception('Запрещено перезаписывать аккаунт');
            }
        } else if ($name === 'browser') {
            if (!isset($this->browser) && $value instanceof browser) {
                $this->browser = $value;
                return;
            } else {
                throw new Exception('Запрещено перезаписывать браузер');
            }
        } else if ($name === 'proxy') {
            $this->proxy = $value;
            return;
        } else if ($name === 'captcha') {
            $this->captcha = $value;
            return;
        } else if ($name === 'messages_new') {
            $this->messages_new = (int) $value;
            return;
        }

        // Если свойство не найдено
        throw new Exception('Свойство не найдено: ' . $name);
    }

    /**
     * Прочитать свойство
     *
     * @param string $name Название
     *
     * @return mixed
     */
    public function __get(string $name)
    {
        if ($name === 'id') {
            if (isset($this->id)) {
                return $this->id;
            } else {
                throw new Exception('Идентификатор не инициализирован');
            }
        } else if ($name === 'session') {
            if (isset($this->session)) {
                return $this->session;
            } else {
                throw new Exception('Сессия не инициализирована');
            }
        } else if ($name === 'key') {
            if (isset($this->key)) {
                return $this->key;
            } else {
                throw new Exception('Ключ не инициализирован');
            }
        } else if ($name === 'account') {
            if (isset($this->account)) {
                return $this->account;
            } else {
                throw new Exception('Аккаунт не инициализирован');
            }
        } else if ($name === 'version') {
            if (isset($this->version)) {
                return $this->version;
            } else {
                throw new Exception('Версия не инициализирована');
            }
        } else if ($name === 'browser') {
            return $this->browser ?? $this->browser = new browser([
                'base_uri'  => 'https://api.vk.com/method/',
                'cookies'   => true
            ]);
        } else if ($name === 'proxy') {
            return $this->proxy;
        } else if ($name === 'captcha') {
            return $this->captcha;
        } else if ($name === 'messages_new') {
            return $this->messages_new;
        }

        throw new Exception('Свойство не найдено: ' . $name);
    }

    /**
     * Проверить свойство на инициализированность
     *
     * @param string $name Название
     *
     * @return mixed
     */
    public function __isset(string $name)
    {
        if ($name === 'id') {
            return isset($this->id);
        } else if ($name === 'session') {
            return isset($this->session);
        } else if ($name === 'key') {
            return isset($this->key);
        } else if ($name === 'account') {
            return isset($this->account);
        } else if ($name === 'version') {
            return isset($this->version);
        } else if ($name === 'browser') {
            return isset($this->browser);
        } else if ($name === 'proxy') {
            return isset($this->proxy);
        } else if ($name === 'captcha') {
            return isset($this->captcha);
        } else if ($name === 'messages_new') {
            return isset($this->messages_new);
        }

        throw new Exception('Свойство не найдено: ' . $name);
    }

    /**
     * Вызвать метод
     *
     * Ищет класс описывающий метод API ВКонтакте,
     * создаёт и возвращает его объект
     *
     * @param string $method Метод
     * @param array $params Параметры
     *
     * @return method
     */
    public function __call(string $method, array $params): method
    {
        if (class_exists($class = '\\hood\\vk\\api\\methods\\' . $method . 's')) {
            // Если найден класс реализующий запрошенный метод
            return new $class($this, ...$params);
        }

        throw new Exception('Метод не найден: ' . $method);
    }

    /**
     * Вызвать статический метод
     *
     * Ищет класс описывающий метод API ВКонтакте,
     * создаёт и возвращает его объект
     *
     * @param string $method Метод
     * @param array $params Параметры
     *
     * @return method
     */
    public static function __callStatic(string $method, array $params): method
    {
        if (class_exists($class = '\\hood\\vk\\api\\methods\\' . $method . 's')) {
            return $class(self, ...$params);
        }

        throw new Exception('Метод не найден: ' . $method);
    }

    /**
     * Конвертировать в строку
     *
     * @return string
     */
    public function __toString(): string
    {
        return (string) $this->id;
    }
}
